#include <CapacitiveSensor.h>

/*
 * CapitiveSense Library Demo Sketch
 * Paul Badger 2008
 * Uses a high value resistor e.g. 10M between send pin and receive pin
 * Resistor effects sensitivity, experiment with values, 50K - 50M. Larger resistor values yield larger sensor values.
 * Receive pin is the sensor pin - try different amounts of foil/metal on this pin
 */
#define DEBUG_ON

#define led_blu  10
#define led_gre  11
#define led_yel  12
#define led_red  13

#define LDR_top   A0
#define LDR_bot   A1

#define LDR_sensorMin 20
#define LDR_sensorNor 250
#define LDR_sensorMax 500 

#define LDR_switchValue 180
#define Touch_switchValue 1500
#define Delay_NextState 1000

CapacitiveSensor   cs_3_9 = CapacitiveSensor(3,9);        // 10M resistor between pins 4 & 2, pin 2 is sensor pin, add a wire and or foil if desired
CapacitiveSensor   cs_3_8 = CapacitiveSensor(3,8);        // 10M resistor between pins 4 & 6, pin 6 is sensor pin, add a wire and or foil
CapacitiveSensor   cs_3_7 = CapacitiveSensor(3,7);        // 10M resistor between pins 4 & 8, pin 8 is sensor pin, add a wire and or foil

CapacitiveSensor   cs_2_6 = CapacitiveSensor(2,6);        // 10M resistor between pins 4 & 2, pin 2 is sensor pin, add a wire and or foil if desired
CapacitiveSensor   cs_2_5 = CapacitiveSensor(2,5);        // 10M resistor between pins 4 & 6, pin 6 is sensor pin, add a wire and or foil
CapacitiveSensor   cs_2_4 = CapacitiveSensor(2,4);        // 10M resistor between pins 4 & 8, pin 8 is sensor pin, add a wire and or foil

int aSensorTop = 0;
int aSensorBot = 0;

long total1 = 0;
long total2 = 0;
long total3 = 0;
long total4 = 0;
long total5 = 0;
long total6 = 0;

int STATE = 0;
bool bBlu = false;
bool bGre = false;
bool bYel = false;
bool bRed = false;

long randNumber;
bool bRand_active = false;
int iRand_som = 15;
int iRand_min;
int iRand_max;

void setup()                    
{
   //cs_3_9.set_CS_AutocaL_Millis(0xFFFFFFFF);     // turn off autocalibrate on channel 1 - just as an example
   Serial.begin(9600);
   pinMode(LED_BUILTIN, OUTPUT);
   pinMode(led_blu, OUTPUT);
   pinMode(led_gre, OUTPUT);
   pinMode(led_yel, OUTPUT);
   pinMode(led_red, OUTPUT);

      
   digitalWrite(led_gre,LOW);   
   digitalWrite(led_yel,LOW);   
   digitalWrite(led_red,LOW);   
}

void readData()
{
    total1 =  cs_3_9.capacitiveSensor(30);   // BLUE
    total2 =  cs_3_8.capacitiveSensor(30);   // GREEN
    total3 =  cs_3_7.capacitiveSensor(30);   // BOTTOM
    
    total4 =  cs_2_6.capacitiveSensor(30);   // YELLOW
    total5 =  cs_2_5.capacitiveSensor(30);   // RED
    total6 =  cs_2_4.capacitiveSensor(30);   // TOP

    aSensorTop = analogRead(LDR_top);             // read the input pin
    aSensorBot = analogRead(LDR_bot);             // read the input pin
}

void DEBUG();
void ALL_ON(int);
void TOGGLE(int,int);
void ALL_BLINK(int);
void TOUCH_COLOURS();

void loop()                    
{    
    readData();  
    
    if(STATE == 0)
    {
      
      if(aSensorTop <= LDR_switchValue && aSensorBot <= LDR_switchValue)
      {
        ALL_BLINK(300);
        STATE = 10;
        delay(Delay_NextState);
      }
    }
    if(STATE == 10)
    {
      int range = map(aSensorTop, LDR_sensorMin, LDR_sensorMax, 255, 1);
          
      TOGGLE(1,range);  
      TOGGLE(2,range);
      TOGGLE(3,range);
      TOGGLE(4,range);

      if(total6 >= Touch_switchValue && total1 >= Touch_switchValue)
      {
        ALL_BLINK(300);
        STATE = 20;
        delay(Delay_NextState);
      }
    }
    if(STATE == 20)
    {
      ALL_ON(1); 
      ALL_ON(2); 
      ALL_ON(3); 
      ALL_ON(4); 

      if(bBlu && bGre && bYel && bRed)
      {
        bBlu = false;
        bGre = false;
        bYel = false;
        bRed = false;
          
        ALL_BLINK(300);
        STATE = 30;
      }
    }
    if(STATE == 30)
    {
      TOUCH_COLOURS();
      
      if(bBlu && bGre && bYel && bRed)
      {
        bBlu = false;
        bGre = false;
        bYel = false;
        bRed = false;

        iRand_min = 1;
          
        ALL_BLINK(300);
        STATE = 0;
      }
    }

    #ifdef DEBUG_ON
      DEBUG();
    #endif
    delay(10);                             // arbitrary delay to limit data to serial port 
}
/*
 * iRand_som
 * iRand_min;
*  iRand_max;
 */
void TOUCH_COLOURS()
{
  if(bYel)
  {
    iRand_min = 2;
  }
  if(bYel && bRed )
  {
    iRand_min = 3;
  }
  if(bYel && bRed && bGre)
  {
    iRand_min = 4;
  }
  
  if(!bRand_active)
  {   
    randNumber = random(iRand_min,4);
    bRand_active = true;
  }

  
  
  switch(randNumber)
  {
    case 1: 
      digitalWrite(led_yel,HIGH);

      digitalWrite(led_red,LOW);
      digitalWrite(led_gre,LOW);
      digitalWrite(led_blu,LOW);
      break;
    case 2:
      digitalWrite(led_red,HIGH);

      digitalWrite(led_gre,LOW);
      digitalWrite(led_blu,LOW);
      digitalWrite(led_yel,LOW);
      break;
    case 3:
      digitalWrite(led_gre,HIGH);

      digitalWrite(led_blu,LOW);
      digitalWrite(led_yel,LOW);
      digitalWrite(led_red,LOW);
      break;
    case 4:
      digitalWrite(led_blu,HIGH);

      digitalWrite(led_yel,LOW);
      digitalWrite(led_red,LOW);
      digitalWrite(led_gre,LOW);
      break;
  }
  

  if(total4 >= Touch_switchValue && (randNumber == 1))
  {
    bYel = true;
    bRand_active = !bRand_active;
    digitalWrite(led_yel,LOW);
  }

  if(total5 >= Touch_switchValue && (randNumber == 2))
  {
    bRed = true;
    bRand_active = !bRand_active;
    digitalWrite(led_red,LOW);
  }

  if(total2 >= Touch_switchValue && (randNumber == 3))
  {
    bGre = true;
    bRand_active = !bRand_active;
    digitalWrite(led_gre,LOW);
  }

  if(total1 >= Touch_switchValue && (randNumber == 4))
  {
    bBlu = true;
    bRand_active = !bRand_active;
    digitalWrite(led_blu,LOW);
  }
}

void ALL_ON(int led){
  switch(led)
  {
    case 1:                            
      if(total1 >= Touch_switchValue)
      {
        digitalWrite(led_blu,HIGH);
        bBlu = true;
      }
      break;
    case 2:
       if(total2 >= Touch_switchValue)
      {
        digitalWrite(led_gre,HIGH);
        bGre = true;
      }
     break;
    case 3:
      if(total4 >= Touch_switchValue)
      {
        digitalWrite(led_yel,HIGH);
        bYel = true;
      }
      break;
    case 4:
      if(total5 >= Touch_switchValue)
      {
        digitalWrite(led_red,HIGH);
        bRed = true;
      }
      break;
  } 

  if(bBlu && bGre && bYel && bRed)
  {
    delay(2000);
    digitalWrite(led_blu,LOW);
    digitalWrite(led_gre,LOW);
    digitalWrite(led_yel,LOW);
    digitalWrite(led_red,LOW);
  }
}

/*
 * TOGGLE LED
 * 1 = BLUE
 * 2 = GREEN
 * 3 = YELLOW
 * 4 = RED
 */
void TOGGLE(int led, int sensitivity)
{
  switch(led)
  {
    case 1:                               
      if(total1 >= Touch_switchValue)
      {
        analogWrite(led_blu,sensitivity);
        //digitalWrite(led_blu,HIGH);
      }
      else{
        analogWrite(led_blu,LOW);
        //digitalWrite(led_blu,LOW);
      }
      break;
    case 2:
      if(total2 >= Touch_switchValue)
      {
        analogWrite(led_gre,sensitivity);
        //digitalWrite(led_gre,HIGH);
      }
      else{
        analogWrite(led_gre,LOW);
        //digitalWrite(led_gre,LOW);
      }
      break;
    case 3:
      if(total4 >= Touch_switchValue)
      {
        digitalWrite(led_yel,HIGH);
      }
      else{
        digitalWrite(led_yel,LOW);
      }
      break;
    case 4:
      if(total5 >= Touch_switchValue)
      {
        digitalWrite(led_red,HIGH);
      }
      else{
        digitalWrite(led_red,LOW);
      }
      break;
  }      
}

/*
 *  BLINK LED
 *  variable delay in seconds
 *  flashes all 4 leds for a x seconds
 */
void ALL_BLINK(int sDelay)
{
    digitalWrite(led_blu,HIGH);
    digitalWrite(led_gre,HIGH);
    digitalWrite(led_yel,HIGH);
    digitalWrite(led_red,HIGH);
    delay(sDelay); 
    digitalWrite(led_blu,LOW);
    digitalWrite(led_gre,LOW);
    digitalWrite(led_yel,LOW);
    digitalWrite(led_red,LOW);
}

/*
 * SERTAIL MONITOR DEBUG
 * shows sensor data
 */
void DEBUG()
{
    Serial.print(STATE);                         // STATE
    Serial.print("\t");
    Serial.print(total1);                         // print sensor output 1
    Serial.print("\t");
    Serial.print(total2);                         // print sensor output 2
    Serial.print("\t");
    Serial.print(total3);                         // print sensor bottom
    Serial.print("\t");
    Serial.print(total4);
    Serial.print("\t");
    Serial.print(total5);
    Serial.print("\t");
    Serial.print(total6);
    Serial.print("\t");
    Serial.print("\t");
    Serial.print(aSensorTop);
    Serial.print("\t");
    Serial.print(aSensorBot);
    Serial.print("\t");
    Serial.print(bBlu);
    Serial.print("\t");
    Serial.print(bGre);
    Serial.print("\t");
    Serial.print(bYel);
    Serial.print("\t");
    Serial.print(bRed);
    Serial.print("\t");
    Serial.println(randNumber);
}
